"""
This module cocntains the nodes that are capable of
building the scene by positioning the inputs in
different positions.
"""
from yta_editor_nodes.compositor.displacement_with_rotation import DisplacementWithRotationNodeCompositor


__all__ = [
    'DisplacementWithRotationNodeCompositor'
]